package footbalmanager;

import java.util.Scanner;

/**
 * Fotbalovy manazer.
 *
 * @author Jakub Trmota
 */
public class Main {

    /**
     * Debugovaci mod.
     */
    private static final boolean DEBUG = false;
    /**
     * Nacitani a ukladani dat.
     */
    private static FileStorage storage = null;
    /**
     * Pole tymu.
     */
    private static Team[] teams = new Team[16];
    /**
     * Pole zapasu
     */
    private static Match[][] matches = new Match[30][8];

    /**
     * Spousteci funkce.
     *
     * @param args argumenty prikazove radky, zadny se neprijima
     */
    public static void main(String[] args) {
        storage = FileStorage.getStorage(DEBUG);

        start();

        System.out.println("Dekuji za zahrani!");
    }

    /**
     * Zobrazi uvodni menu hry.
     */
    private static void start() {
        System.out.println("*********************");
        System.out.println("* Fotbalový manažer *");
        System.out.println("*********************");
        System.out.println("");
        System.out.println("[N]ová hra");
        System.out.println("[P]okračovat");
        System.out.println("[K]onec");
        System.out.println("");
        System.out.print(">>> Vyber si co chceš dělat > ");

        Scanner scanner = new Scanner(System.in);

        while (true) {
            String key = scanner.nextLine();
            if (key.equalsIgnoreCase("n")) {
                newGame();

                break;
            } else if (key.equalsIgnoreCase("p")) {
                continueGame();

                break;
            } else if (key.equalsIgnoreCase("k")) {
                return;
            } else {
                System.out.print(">>> Musíš zadat N, P nebo K!!! Vyber si co chceš dělat > ");
            }
        }
    }

    /**
     * Zalozi novou hru.
     */
    private static void newGame() {
        // Nacte ze souboru tymy
        if (!storage.loadTeams(teams)) {
            System.out.println("\n!!! Nepodařilo se načíst týmy soutěže, hra se teď ukončí...\n");

            return;
        }

        // Nacte ze souboru rozpis
        if (!storage.loadMatches(matches)) {
            System.out.println("\n!!! Nepodařilo se načíst rozlosování soutěže, hra se teď ukončí...\n");

            return;
        }

        // Vypise tymy
        System.out.println(" ID  Název                     Útok/Obrana");
        System.out.println("------------------------------------------");
        for (int i = 0; i < 16; i++) {
            System.out.printf("[%2d] %-25s (%.0f/%.0f)\n", i + 1, teams[i].getName(), teams[i].getOfence(), teams[i].getDefence());
        }
        System.out.println("------------------------------------------");
        System.out.print(">>> Vyber si za jaký tým chceš hrát (0 = hraje jenom počítač) > ");

        // Necha vybrat tym
        Scanner input = new Scanner(System.in);

        int team = -1;
        while (true) {
            try {
                String teamString = input.next();
                team = new Integer(teamString).intValue();
            } catch (Exception e) {
                if (DEBUG) {
                    e.printStackTrace();
                }

                team = -1;
            }

            if ((team >= 0) && (team <= 16)) {
                break;
            } else {
                System.out.print(">>> Musíš zadat číslo týmu (0 - 16) > ");
            }
        }

        // Zapise tym
        if (team == 0) {
            System.out.println("Vybral jsi, že ligu bude simulovat pouze počítač.");
        } else {
            teams[team - 1].setUser(true);
            System.out.println("\nTvůj tým je " + teams[team - 1].getName() + ".\n");
        }

        // Ulozi hru
        if (!storage.saveLeague(teams, matches)) {
            System.out.println("\n!!! Nepodařilo se uložit hru, hra se teď ukončí...\n");

            return;
        }

        play();
    }

    /**
     * Pokracuje v rozehrane hre.
     */
    private static void continueGame() {
        if (!storage.loadLeague(teams, matches)) {
            System.out.println("\n!!! Nepodařilo se načíst hru, budete pokračovat založením nové hry...\n");

            newGame();

            return;
        }

        play();
    }

    /**
     * Hraje vsech 30 kol ligy.
     */
    private static void play() {
        int round = teams[0].getMatches();
        if ((round > 0) && (round < 30)) {
            round++;
        }

        // Konec sezony
        if (round == 30) {
            System.out.println("\nTato sezona je již dohrána.\n");
            showTable();
        } else {
            for (; round <= 30; round++) {
                // Pouze info
                if (round == 0) {
                    System.out.println("\nZačíná podzimní část ligy...\n");
                } else if (round == 16) {
                    System.out.println("\nZačíná jarní část ligy...\n");
                } else if (round == 30) {
                    System.out.println("\nPoslední kolo...\n");
                }

                if (round == 0) {
                    System.out.println("----------------------------------------------------------------------------");
                    System.out.println("- Tabulka před 1. kolem ----------------------------------------------------");
                    showTable();
                } else {
                    // Simulovat zapasy
                    for (int match = 0; match < 8; match++) {
                        simulateMatch(round - 1, match);
                    }

                    System.out.println("----------------------------------------------------------------------------");
                    System.out.printf("- Tabulka po %-2s. kole ------------------------------------------------------\n", round);
                    showTable();
                    showResults(round - 1);
                }

                if (!storage.saveLeague(teams, matches)) {
                    System.out.println("\n!!! Hru se nepodařilo uložit! Můžete pokračovat ve hraní, ale při ukončení hry přijdete o dosažené výsledky!\n");
                }

                System.out.print("\n>>> Zadej cokoliv pro pokračování nebo [K] pro konec > ");
                try {
                    switch (System.in.read()) {
                        case 75: // K
                        case 107: // k
                            return;
                    }

                } catch (Exception e) {
                    if (DEBUG) {
                        e.printStackTrace();
                    }
                }
            }
            // Konec ligy
        }
    }

    /**
     * Simuluje jeden zapas.
     *
     * @param round kolo
     * @param match zapas
     */
    private static void simulateMatch(int round, int match) {
        SimulateMatch sm = new SimulateMatch(round, matches[round][match], teams);
        sm.simulate();
    }

    /**
     * Zobrazi tabulku.
     */
    private static void showTable() {
        int[] points = new int[16];
        int[] order = new int[16];

        for (int i = 0; i < 16; i++) {
            points[i] = teams[i].getSortPoints();
        }

        int max = -1;
        int position = -1;
        for (int i = 0; i < 16; i++) {
            for (int x = 0; x < 16; x++) {
                if (points[x] > max) {
                    max = points[x];
                    position = x;
                }
            }
            order[i] = position;
            points[position] = -1;

            max = -1;
            position = -1;
        }

        System.out.println("----------------------------------------------------------------------------");
        System.out.println("- Pořadí  Název                    Zápasů Výhry Remízy Prohry Skóre   Body -");
        System.out.println("----------------------------------------------------------------------------");
        for (int i = 0; i < 16; i++) {
            int x = order[i];

            System.out.printf("%s %6d. %-25s %5d %5d %6d %6d %3d:%-3d %4d\n", (teams[x].isUser() ? "*" : " "), (i + 1), teams[x].getName(), teams[x].getMatches(), teams[x].getWins(), teams[x].getTies(), teams[x].getLooses(), teams[x].getPGoals(), teams[x].getNGoals(), teams[x].getPoints());
        }
        System.out.println("----------------------------------------------------------------------------");
    }

    /**
     * Zobrazi vysledky.
     *
     * @param round kolo
     */
    private static void showResults(int round) {
        System.out.printf("- %-2s. kolo -----------------------------------------------------------------\n", (round + 1));
        System.out.println("----------------------------------------------------------------------------");

        for (int i = 0; i < 8; i++) {
            System.out.printf("%s %25s - %-25s %3d:%-3d\n", ((teams[matches[round][i].teamHome()].isUser() || teams[matches[round][i].teamVisitor()].isUser()) ? "*" : " "), teams[matches[round][i].teamHome()].getName(), teams[matches[round][i].teamVisitor()].getName(), matches[round][i].scoreHome(), matches[round][i].scoreVisitor());
        }

        System.out.println("----------------------------------------------------------------------------");
    }
}
