package footbalmanager;

import java.util.Scanner;
import java.util.Random;

/**
 * Trida, ktera simuluje jeden konkretni zapas.
 *
 * @author Jakub Trmota
 */
public class SimulateMatch {

    /**
     * Aktualni zapas, ktery se simuluje.
     */
    private Match match = null;
    /**
     * Tym domacich.
     */
    private Team home = null;
    /**
     * Tym hostu.
     */
    private Team visitor = null;
    /**
     * Kolo, ktere se hraje.
     */
    private int round = 0;
    /**
     * Pocet golu pro domaci.
     */
    private int scoreHome = 0;
    /**
     * Pocet golu pro hosty.
     */
    private int scoreVisitor = 0;
    /**
     * Koeficient domaciho utoku.
     */
    private double homeOfence = 0;
    /**
     * Koeficient domaci obrany.
     */
    private double homeDefence = 0;
    /**
     * Koeficient hostujiciho utoku.
     */
    private double visitorOfence = 0;
    /**
     * Koeficient hostujici obrany.
     */
    private double visitorDefence = 0;
    /**
     * Urcuje, zda hraje uzivatel (true) nebo pocitac (false).
     */
    private boolean isUser = false;
    /**
     * Generator nahodnych cisel.
     */
    private Random random = null;

    /**
     * Konstruktor tridy.
     *
     * @param round kolo, ktere se hraje
     * @param match zapas, ktery se hraje
     * @param teams pole vsech tymu
     */
    public SimulateMatch(int round, Match match, Team[] teams) {
        this.round = round;
        this.match = match;
        this.home = teams[match.teamHome()];
        this.visitor = teams[match.teamVisitor()];

        this.random = new Random();
    }

    /**
     * Odsimiluje nastaveny zapas.
     */
    public void simulate() {
        int ball = 0;
        int firstBall = 0;

        // PRVNI POLOCAS

        // Nastav taktiku pro prvni polocas
        this.setTactics(1);

        // Kdo ma balon - 1 = domaci / 2 = hoste
        ball = (random.nextInt() % 2) + 1;
        firstBall = ball;

        for (int minute = 1; minute <= 45; minute++) {
            ball = this.play(ball, minute);
        }

        // DRUHY POLOCAS

        // Nastav taktiku pro druhy polocas
        this.setTactics(2);

        // Rozehrava druhy tym nez v prvnim polocase
        ball = (firstBall == 1) ? 2 : 1;

        for (int minute = 46; minute <= 90; minute++) {
            ball = this.play(ball, minute);
        }

        // PO ZAPASE

        // Ulozeni vysledku zapasu
        this.setScoreOfMatch(this.scoreHome, this.scoreVisitor);

        // Uprava koeficientu tymu
        this.setTeamsCoeficients(this.scoreHome, this.scoreVisitor);
    }

    /**
     * Nastavi taktiku a koeficienty pro domaci a hostujici tym.
     *
     * @param half pro jaky polocas
     */
    private void setTactics(int half) {
        // Nastaveni domaciho prostredi
        this.homeOfence = this.home.getOfence() + 2;
        this.homeDefence = this.home.getDefence() - 1;
        this.visitorOfence = this.visitor.getOfence() - 1;
        this.visitorDefence = this.visitor.getDefence() + 2;

        // Taktika
        int tacticsHome = 0;
        int tacticsVisitor = 0;
        int tacticsUser = 0;

        if (this.home.isUser() || this.visitor.isUser()) {
            this.isUser = true;

            if (half == 1) {
                System.out.printf("- %-2s. kolo -----------------------------------------------------------------\n", (this.round + 1));
                System.out.println("----------------------------------------------------------------------------");
                System.out.printf("- %25s - %-25s --------------------\n", this.home.getName(), this.visitor.getName());
                System.out.println("----------------------------------------------------------------------------\n");
            } else {
                System.out.println("----------------------------------------------------------------------------");
                System.out.println("- Výsledek po prvním poločase ----------------------------------------------");
                System.out.printf("- %3d:%-3d ------------------------------------------------------------------\n", this.scoreHome, this.scoreVisitor);
                System.out.println("----------------------------------------------------------------------------\n");
            }

            System.out.print(">>> Zadej taktiku " + half + ". poločasu ([1] pro absolutní obranu, [2], [3], [4] a [5] pro absolutní útok)  > ");

            Scanner input = new Scanner(System.in);

            while (true) {
                try {
                    String tacticsUserString = input.next();
                    tacticsUser = new Integer(tacticsUserString).intValue();
                } catch (Exception e) {
                    tacticsUser = 0;
                }

                if ((tacticsUser >= 1) && (tacticsUser <= 5)) {
                    break;
                } else {
                    System.out.print(">>> Musíš zadat číslo 1 - 5 > ");
                }
            }
        }

        if (this.home.isUser()) {
            tacticsHome = tacticsUser;
        } else {
            tacticsHome = (half == 1) ? getComputerTacticsFirstHalf(this.homeOfence, this.homeDefence) : getComputerTacticsSecondHalf(this.scoreHome, this.scoreVisitor);
        }

        this.setTacticsOfenceDefence(tacticsHome, 1);

        if (this.visitor.isUser()) {
            tacticsVisitor = tacticsUser;
        } else {
            tacticsVisitor = (half == 1) ? getComputerTacticsFirstHalf(this.visitorOfence, this.visitorDefence) : getComputerTacticsSecondHalf(this.scoreVisitor, this.scoreHome);
        }

        this.setTacticsOfenceDefence(tacticsVisitor, 2);
    }

    /**
     * Nastavi koeficienty utoku a obrany pro zvolenou taktiku.
     *
     * @param tactics zvolena taktika (1-5)
     * @param team tym domaci (1), hoste (2)
     */
    private void setTacticsOfenceDefence(int tactics, int team) {
        int ofence = 0;
        int defence = 0;

        if (tactics == 1) {
            ofence = -2;
            defence = 4;
        } else if (tactics == 2) {
            ofence = -1;
            defence = 2;
        } else if (tactics == 3) {
        } else if (tactics == 4) {
            ofence = 2;
            defence = -1;
        } else if (tactics == 5) {
            ofence = 4;
            defence = -2;
        }

        if (team == 1) { // Domaci
            this.homeOfence += ofence;
            this.homeDefence += defence;
        } else { // Hoste
            this.visitorOfence += ofence;
            this.visitorDefence += defence;
        }
    }

    /**
     * Urci taktiku pro pocitac pro prvni polocas.
     *
     * @param ofence koeficient utoku
     * @param defence koeficient obrany
     * @return cislo taktiky (1-5)
     */
    private int getComputerTacticsFirstHalf(double ofence, double defence) {
        if ((ofence - defence) > 10) {
            return 5;
        } else if ((ofence - defence) > 5) {
            return 4;
        } else if ((ofence - defence) < 5) {
            return 2;
        } else if ((ofence - defence) < 10) {
            return 1;
        } else {
            return 3;
        }
    }

    /**
     * Urci taktiku pro pocitac pro druhy polocas.
     *
     * @param pgoals pocet vstrelenych golu
     * @param ngoals pocet obdrzenych golu
     * @return cislo taktiky (1-5)
     */
    private int getComputerTacticsSecondHalf(int pgoals, int ngoals) {
        if ((pgoals - ngoals) >= 2) {
            return 1;
        } else if ((pgoals - ngoals) == 1) {
            return 2;
        } else if (pgoals == ngoals) {
            return 3;
        } else if ((pgoals - ngoals) == -1) {
            return 4;
        } else {
            return 5;
        }
    }

    /**
     * Hraje 1 minutu zapasu a vrati, ktery tym ma po ni mic.
     *
     * @param ball ktery tym ma nyni mic, 1 domaci, 2 hoste
     * @param minute minuta, ktera se hraje
     * @return cislo muzstva, ktere ma mic
     */
    private int play(int ball, int minute) {
        double ballHome = 0;
        double ballVisitor = 0;

        if (ball == 1) { // utoci domaci
            ballHome = random.nextDouble() * homeOfence;
            ballVisitor = random.nextDouble() * visitorDefence;

            if (((ballHome - ballVisitor) > 16) && ((ballHome - ballVisitor) < 18)) {
                scoreHome++;
                if (this.isUser) {
                    System.out.println(minute + ". minuta zápasu - " + this.home.getName() + " vstřílí gól - stav zápasu " + scoreHome + ":" + scoreVisitor);
                }
                return 2;
            } else if (ballHome < ballVisitor) {
                return 2;
            }
            return 1;
        } else { // utoci hoste
            ballHome = random.nextDouble() * homeDefence;
            ballVisitor = random.nextDouble() * visitorOfence;

            if (((ballVisitor - ballHome) > 16) && ((ballVisitor - ballHome) < 18)) {
                scoreVisitor++;
                if (this.isUser) {
                    System.out.println(minute + ". minuta zápasu - " + this.visitor.getName() + " vstřílí gól - stav zápasu " + scoreHome + ":" + scoreVisitor);
                }
                return 1;
            } else if (ballVisitor < ballHome) {
                return 1;
            }
            return 2;
        }
    }

    /**
     * Ulozi skore zapasu do tabulky a zapasu.
     *
     * @param scoreHome skore domaci
     * @param scoreVisitor skore hoste
     */
    private void setScoreOfMatch(int scoreHome, int scoreVisitor) {
        this.match.setScore(scoreHome, scoreVisitor);
        if (scoreHome > scoreVisitor) {
            this.home.addWin();
            this.home.addGoals(scoreHome, scoreVisitor);
            this.visitor.addLoose();
            this.visitor.addGoals(scoreVisitor, scoreHome);
        } else if (scoreHome < scoreVisitor) {
            this.home.addLoose();
            this.home.addGoals(scoreHome, scoreVisitor);
            this.visitor.addWin();
            this.visitor.addGoals(scoreVisitor, scoreHome);
        } else {
            this.home.addTie();
            this.home.addGoals(scoreHome, scoreVisitor);
            this.visitor.addTie();
            this.visitor.addGoals(scoreVisitor, scoreHome);
        }
    }

    /**
     * Upravi koeficienty tymu.
     *
     * @param scoreHome skore domaci
     * @param scoreVisitor skore hoste
     */
    private void setTeamsCoeficients(int scoreHome, int scoreVisitor) {
        if (scoreHome == 0) {
            this.home.addOfence(-0.1);
            this.visitor.addDefence(0.1);
        } else if (scoreHome >= 3) {
            this.home.addOfence(0.1);
            this.visitor.addDefence(-0.1);
        }

        if (scoreVisitor == 0) {
            this.visitor.addOfence(-0.1);
            this.home.addDefence(0.1);
        } else if (scoreVisitor >= 3) {
            this.visitor.addOfence(0.1);
            this.home.addDefence(-0.1);
        }
    }
}
