package footbalmanager;

import java.io.Serializable;

/**
 * Trida pro ulozeni vsech informaci o tymu.
 *
 * @author Jakub Trmota
 */
public class Team implements Serializable {

    /**
     * Nazev tymu.
     */
    private String name = "";
    /**
     * Tym ovlada uzivatel.
     */
    private boolean user = false;
    /**
     * Koeficient utoku.
     */
    private double ofence = 0;
    /**
     * Koeficient obrany.
     */
    private double defence = 0;
    /**
     * Pocet zapasu.
     */
    private int matches = 0;
    /**
     * Pocet vyher.
     */
    private int wins = 0;
    /**
     * Pocet proher.
     */
    private int looses = 0;
    /**
     * Pocer remiz.
     */
    private int ties = 0;
    /**
     * Pocet vstrelenych golu.
     */
    private int pgoals = 0;
    /**
     * Pocet obdrzenych golu.
     */
    private int ngoals = 0;
    /**
     * Pocet bodu.
     */
    private int points = 0;
    /**
     * Pocet bodu za prohru.
     */
    public static final int LOOSE = 0;
    /**
     * Pocet bodu za remizu.
     */
    public static final int TIE = 1;
    /**
     * Pocet bodu za vyhru.
     */
    public static final int WIN = 3;

    /**
     * Konstruktor, vytrvori tym pomoci jmeno a nastavni koeficient utoku a obrany.
     *
     * @param name nazev tymu
     * @param ofence koeficient utoku
     * @param defence koeficient obrany
     */
    public Team(String name, int ofence, int defence) {
        this.name = name;
        this.ofence = ofence;
        this.defence = defence;
    }

    /**
     * Vrati nazev tymu.
     *
     * @return nazev tymu
     */
    public String getName() {
        return this.name;
    }

    /**
     * Vrati jestli tym ovlada uzivatel.
     *
     * @return true, pokud tym ovlada uzivatel, jinak false
     */
    public boolean isUser() {
        return this.user;
    }

    /**
     * Nastavi ovladani tymu uzivatelem.
     *
     * @param user true, pokud ma tym ovladat uzivatel, jinak false
     */
    public void setUser(boolean user) {
        this.user = user;
    }

    /**
     * Vrati koeficient utoku.
     *
     * @return koeficient utoku
     */
    public double getOfence() {
        return this.ofence;
    }

    /**
     * Prida ke koeficientu utoku.
     *
     * @param ofence koeficient utoku
     */
    public void addOfence(double ofence) {
        this.ofence += ofence;
    }

    /**
     * Vrati koeficient obrany.
     *
     * @return koeficient obrany
     */
    public double getDefence() {
        return this.defence;
    }

    /**
     * Prida koeficient obrany.
     *
     * @param defence koeficient obrany
     */
    public void addDefence(double defence) {
        this.defence += defence;
    }

    /**
     * Vrati pocet odehranych zapasu.
     *
     * @return pocet odehranych zapasu
     */
    public int getMatches() {
        return this.matches;
    }

    /**
     * Vrati pocet vyher.
     *
     * @return pocer vyher
     */
    public int getWins() {
        return this.wins;
    }

    /**
     * Prida vyhru.
     */
    public void addWin() {
        this.wins++;
        this.matches++;
        this.points += WIN;
    }

    /**
     * Vrati pocet proher.
     *
     * @return pocet proher
     */
    public int getLooses() {
        return this.looses;
    }

    /**
     * Prida prohru.
     */
    public void addLoose() {
        this.looses++;
        this.matches++;
        this.points += LOOSE;
    }

    /**
     * Vrati pocet remiz.
     *
     * @return pocet remiz
     */
    public int getTies() {
        return this.ties;
    }

    /**
     * Prida remizu.
     */
    public void addTie() {
        this.ties++;
        this.matches++;
        this.points += TIE;
    }

    /**
     * Vrati pocet vstrelenych golu.
     *
     * @return pocet vstrelenych golu
     */
    public int getPGoals() {
        return this.pgoals;
    }

    /**
     * Vrati pocet obdrzenych golu.
     *
     * @return pocet obdrzenych golu
     */
    public int getNGoals() {
        return this.ngoals;
    }

    /**
     * Prida vstrelene a obdrzene goly.
     *
     * @param pgoals pocet vstrelenych golu
     * @param ngoals pocet obdrzenych golu
     */
    public void addGoals(int pgoals, int ngoals) {
        this.pgoals += pgoals;
        this.ngoals += ngoals;
    }

    /**
     * Vrati pocet bodu.
     *
     * @return pocet bodu
     */
    public int getPoints() {
        return this.points;
    }

    /**
     * Vrati pocet pobud pro serazeni. Bere vpotaz, ze nejpreve se porovnaji body a pak brankovy rozdil, takze bodum nastavuje vetsi vahu.
     *
     * @return pocet bodu pro serazeni
     */
    public int getSortPoints() {
        return 1000000 + (this.points * 10000) + (this.pgoals - this.ngoals);
    }
}
